#! /usr/bin/env python
# -*- coding: utf-8 -*-
'''
Created on 2023/8/6 18:29:31

@author: LiuKuan
@copyright: Apache License, Version 2.0
'''
import matplotlib.pyplot as plt
# ===============================================================================
r'''
'''
# ===============================================================================

axes_title = plt.title

axes_legend = plt.legend

axes_xlabel = plt.xlabel
axes_ylable = plt.ylabel

axes_xlim = plt.xlim
axes_ylim = plt.ylim

axes_xscale = plt.xscale
axes_yscale = plt.yscale

axes_xticks = plt.xticks
axes_yticks = plt.yticks


